/*
 * Common viewport-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "viewport.h"



/* Initialise the origin for a viewport. */

void viewport_set_origin(viewport_t *v, int xorigin, int yorigin)
{
    int frame;

    /* Set the origin for every frame. */

    v->xorigin = xorigin;
    v->yorigin = yorigin;

    for (frame = 0; frame < v->cfg->frames; frame++)
    {
        v->xorigins[frame] = xorigin;
        v->yorigins[frame] = yorigin;
    }
}

/* Update the origin by the given changes in origin coordinates. */

void viewport_update_origin(viewport_t *v, int dx, int dy)
{
    int xstep, ystep;
    int frame = v->cfg->frame;

    /* Change the origin coordinates. */

    v->xorigin += dx;
    v->yorigin += dy;

    /* Determine the magnitude of the scrolling required from this frame to the
       current origin. */

    xstep = v->xorigin - v->xorigins[frame];
    ystep = v->yorigin - v->yorigins[frame];

    /* Scroll the frame, converting to columns and rows. */

    display_scroll(v->cfg, xstep / v->xscale, ystep / v->yscale);

    /* For horizontal scrolling, plot the exposed column at the left (if
       scrolling left) or at the right (if scrolling right). */

    if (v->update)
        v->update(v->xorigin, v->yorigin, xstep, ystep);

    /* Record the origin for this frame. */

    v->xorigins[frame] = v->xorigin;
    v->yorigins[frame] = v->yorigin;
}

/* Update the display, unplotting a sprite and replotting the background. */

void viewport_unplot_sprite_from_image(viewport_t *v, sprite_t *s, sprite_t *bg)
{
    int frame = v->cfg->frame;

    image_unplot_sprite_from_image(s, bg, v->xorigins[frame], v->yorigins[frame]);
}
